
/**
 * section: xmlWriter
 * synopsis: use various APIs for the xmlWriter
 * purpose: tests a number of APIs for the xmlWriter, especially
 *          some methods to write DTD elements.
 * usage: testWriter2
 * test: testWriter2 ; for i in 1 2 ; do diff writerdtd$$i.org writer$$i.res ; done ; rm writer*.res
 * author: Alfred Mickautsch
 * copy: see Copyright for the status of this software.
 */
#include <stdio.h>
#include <string.h>
#include <libxml/encoding.h>
#include <libxml/xmlwriter.h>

#if defined(LIBXML_WRITER_ENABLED) && defined(LIBXML_OUTPUT_ENABLED)

#define MY_ENCODING "ISO-8859-1"

void testXmlwriterDTD1(const char *uri);
void testXmlwriterDTD2(const char *uri);

int
main(void)
{
    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION
        /* first, a DTD with an internal subset */
        testXmlwriterDTD1("writerdtd1.res");

    /* second, an external subset of a DTD */
    testXmlwriterDTD2("writerdtd2.res");

    /*
     * Cleanup function for the XML library.
     */
    xmlCleanupParser();

    /*
     * this is to debug memory for regression tests
     */
    xmlMemoryDump();
    return 0;
}

/**
 * testXmlwriterDTD1:
 * @uri: the output URI
 *
 * test the xmlWriter interface when writing a DTD with an internal subset
 */
void
testXmlwriterDTD1(const char *uri)
{
    int rc;
    xmlTextWriterPtr writer;

    /* Create a new XmlWriter for uri, with no compression. */
    writer = xmlNewTextWriterFilename(uri, 0);
    if (writer == NULL) {
        printf("testXmlwriterDTD1: Error creating the xml writer\n");
        return;
    }

    /* Set indentation on for this writer */
    xmlTextWriterSetIndent(writer, 1);

    /* Start the document with the xml default for the version,
     * encoding ISO 8858-1 and the default for the standalone
     * declaration. */
    rc = xmlTextWriterStartDocument(writer, NULL, MY_ENCODING, NULL);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartDocument\n");
        return;
    }

    /* Start a DTD named "doc" */
    rc = xmlTextWriterStartDTD(writer, BAD_CAST "doc", NULL, NULL);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartDTD\n");
        return;
    }

    /* Write a DTD element named "doc" with content "(src | dest)*" */
    rc = xmlTextWriterWriteDTDElement(writer, BAD_CAST "doc",
                                      BAD_CAST "(src | dest)*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    /* Write a DTD element named "src" with content "EMPTY" */
    rc = xmlTextWriterWriteDTDElement(writer, BAD_CAST "src",
                                      BAD_CAST "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    /* Write a DTD element named dest with content "EMPTY" */
    rc = xmlTextWriterWriteDTDElement(writer, BAD_CAST "dest",
                                      BAD_CAST "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    /* Write an attribute list for element "src" */
    rc = xmlTextWriterWriteDTDAttlist(writer, BAD_CAST "src",
                                      BAD_CAST "ref IDREF #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    /* Write an attribute list for element "dest" */
    rc = xmlTextWriterWriteDTDAttlist(writer, BAD_CAST "dest",
                                      BAD_CAST "id ID #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    /* End the DTD named "doc" */
    rc = xmlTextWriterEndDTD(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterEndDTD\n");
        return;
    }

    /* Start an element named "doc". Since thist is the first
     * element, this will be the root element of the document. */
    rc = xmlTextWriterStartElement(writer, BAD_CAST "doc");
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartElement\n");
        return;
    }

    /* Start an element named "src" as child of "doc". */
    rc = xmlTextWriterStartElement(writer, BAD_CAST "src");
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartElement\n");
        return;
    }

    /* Add an attribute with name "ref" and value "foo" to src. */
    rc = xmlTextWriterWriteAttribute(writer, BAD_CAST "ref",
                                     BAD_CAST "foo");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteAttribute\n");
        return;
    }

    /* End element "src". */
    rc = xmlTextWriterEndElement(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterEndElement\n");
        return;
    }

    /* Start an element named "dest" as child of "doc". */
    rc = xmlTextWriterStartElement(writer, BAD_CAST "dest");
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartElement\n");
        return;
    }

    /* Add an attribute with name "id" and value "foo" to dest. */
    rc = xmlTextWriterWriteAttribute(writer, BAD_CAST "id",
                                     BAD_CAST "foo");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteAttribute\n");
        return;
    }

    /* End element "dest". */
    rc = xmlTextWriterEndElement(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterEndElement\n");
        return;
    }

    /* Start an element named "src" as child of "doc". */
    rc = xmlTextWriterStartElement(writer, BAD_CAST "src");
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterStartElement\n");
        return;
    }

    /* Add an attribute with name "ref" and value "foo" to src. */
    rc = xmlTextWriterWriteAttribute(writer, BAD_CAST "ref",
                                     BAD_CAST "foo");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD1: Error at xmlTextWriterWriteAttribute\n");
        return;
    }

    /* Here we could close the elements src and doc using the
     * function xmlTextWriterEndElement, but since we do not want to
     * write any other elements, we simply call xmlTextWriterEndDocument,
     * which will do all the work. */
    rc = xmlTextWriterEndDocument(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD1: Error at xmlTextWriterEndDocument\n");
        return;
    }

    xmlFreeTextWriter(writer);
}

/**
 * testXmlwriterDTD2:
 * @uri: the output URI
 *
 * test the xmlWriter interface when writing an external subset of a DTD.
 */
void
testXmlwriterDTD2(const char *uri)
{
    int rc;
    xmlTextWriterPtr writer;

    /* Create a new XmlWriter for uri, with no compression. */
    writer = xmlNewTextWriterFilename(uri, 0);
    if (writer == NULL) {
        printf("testXmlwriterDTD2: Error creating the xml writer\n");
        return;
    }

    /* Set indentation on for this writer */
    xmlTextWriterSetIndent(writer, 1);

    /* We do not start a document neither a DTD, since this is
     * an external subset of a DTD.*/

    /* We start with some comment lines... */
    rc = xmlTextWriterWriteComment(writer,
                                   " ............................................................... ");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteComment\n");
        return;
    }

    rc = xmlTextWriterWriteComment(writer,
                                   " xmlwriter-test DTD ............................................ ");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteComment\n");
        return;
    }

    rc = xmlTextWriterWriteComment(writer,
                                   " copied from NewsMLv1.0.dtd in the test/valid/dtds directory.... ");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteComment\n");
        return;
    }

    rc = xmlTextWriterWriteComment(writer,
                                   " ............................................................... ");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteComment\n");
        return;
    }

    /* Some formatting whitespace... */
    rc = xmlTextWriterWriteString(writer, "\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }

    /* Start a parameter entity... */
    rc = xmlTextWriterStartDTDEntity(writer, 1, "assignment");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterStartDTDEntity\n");
        return;
    }

    /* ...write the entity content... */
    rc = xmlTextWriterWriteString(writer, "AssignedBy  CDATA  #IMPLIED\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterWriteString(writer,
                                  "                       Importance  CDATA  #IMPLIED\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterWriteString(writer,
                                  "                       Confidence  CDATA  #IMPLIED\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterWriteString(writer,
                                  "                       HowPresent  CDATA  #IMPLIED\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterWriteString(writer,
                                  "                       DateAndTime CDATA  #IMPLIED");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }

    /* and end the entity. */
    rc = xmlTextWriterEndDTDEntity(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterEndDTDEntity\n");
        return;
    }

    /* Of course, entities can be written with a conveniency function.
     * This shows how internal entities are written. */
    rc = xmlTextWriterWriteDTDEntity(writer, 1, "formalname", NULL, NULL,
                                     NULL,
                                     "FormalName CDATA  #REQUIRED\n"
                                     "                       Vocabulary CDATA  #IMPLIED\n"
                                     "                       Scheme     CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDEntity\n");
        return;
    }

    rc = xmlTextWriterWriteDTDEntity(writer, 1, "localid", NULL, NULL,
                                     NULL,
                                     "Duid ID     #IMPLIED\n"
                                     "                    Euid CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDEntity\n");
        return;
    }

    rc = xmlTextWriterWriteDTDEntity(writer, 1, "data", NULL, NULL, NULL,
                                     "(Encoding\n"
                                     "                 | DataContent )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDEntity\n");
        return;
    }

    rc = xmlTextWriterWriteDTDEntity(writer, 1, "party", NULL, NULL, NULL,
                                     "(Comment*\n"
                                     "                  , Party+ )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDEntity\n");
        return;
    }

    /* Let us write some DTD elements with their attribute lists... */
    rc = xmlTextWriterWriteDTDElement(writer, "AdministrativeMetadata",
                                      "(Catalog? , FileName? , SystemIdentifier? , Provider? , Creator? , Source* , Contributor* , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "AdministrativeMetadata",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    /* Of course, DTD elements and attribute lists can also be written with
     * the start, write string and end functions... */
    rc = xmlTextWriterStartDTDElement(writer, "AssociatedWith");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterStartDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteString(writer, "(Comment* )");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterEndDTDElement(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterEndDTDElement\n");
        return;
    }

    rc = xmlTextWriterStartDTDAttlist(writer, "AssociatedWith");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterStartDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteString(writer, "%localid;\n");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }
    rc = xmlTextWriterWriteString(writer,
                                  "                         NewsItem CDATA  #IMPLIED");
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterWriteString\n");
        return;
    }

    rc = xmlTextWriterEndDTDAttlist(writer);
    if (rc < 0) {
        printf("testXmlwriterDTD2: Error at xmlTextWriterEndDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "BasisForChoice",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "BasisForChoice",
                                      "%localid;\n"
                                      "                         Rank    CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "ByLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "ByLine",
                                      "%localid;\n"
                                      "                 xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Catalog",
                                      "(Resource* , TopicUse* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Catalog",
                                      "%localid;\n"
                                      "                  Href    CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Characteristics",
                                      "(SizeInBytes? , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Characteristics",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Comment", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Comment",
                                      "%localid;\n"
                                      "                  xml:lang      CDATA  #IMPLIED\n"
                                      "                  TranslationOf IDREF  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "ContentItem",
                                      "(Comment* , Catalog? , MediaType? , Format? , MimeType? , Notation? , Characteristics? , %data; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "ContentItem",
                                      "%localid;\n"
                                      "                      Href    CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Contributor", "(%party; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Contributor", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Copyright",
                                      "(Comment* , CopyrightHolder , CopyrightDate )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Copyright",
                                      "%localid;\n"
                                      "                    %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "CopyrightDate",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "CopyrightDate",
                                      "%localid;\n"
                                      "                        xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "CopyrightHolder",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "CopyrightHolder",
                                      "%localid;\n"
                                      "                          xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "CopyrightLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "CopyrightLine",
                                      "%localid;\n"
                                      "                        xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Creator", "(%party; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Creator", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "CreditLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "CreditLine",
                                      "%localid;\n"
                                      "                     xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DataContent", "ANY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DataContent", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DateAndTime", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DateAndTime", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DateId", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DateLabel", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DateLabel", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DateLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DateLine",
                                      "%localid;\n"
                                      "                   xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DefaultVocabularyFor",
                                      "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DefaultVocabularyFor",
                                      "%localid;\n"
                                      "                               Context CDATA  #REQUIRE\n"
                                      "                               Scheme  CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Delete", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Delete",
                                      "%localid;\n"
                                      "                 DuidRef CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DerivedFrom",
                                      "(Comment* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DerivedFrom",
                                      "%localid;\n"
                                      "                      NewsItem CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Description", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Description",
                                      "%localid;\n"
                                      "                      xml:lang CDATA  #IMPLIED\n"
                                      "                      Variant  CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "DescriptiveMetadata",
                                      "(Catalog? , Language* , Genre? , SubjectCode* , OfInterestTo* , TopicOccurrence* , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "DescriptiveMetadata",
                                      "%localid;\n"
                                      "                              %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Encoding", "%data;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Encoding",
                                      "%localid;\n"
                                      "                   Notation CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "EndDate",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "EndDate",
                                      "%localid;\n"
                                      "                  xml:lang CDATA  #IMPLIED\n"
                                      "                  %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "FileName", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "FileName", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "FirstCreated",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "FirstCreated", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "FormalName", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "FormalName",
                                      "%localid;\n"
                                      "                     Scheme  CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Format", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Format",
                                      "%localid;\n"
                                      "                 %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "FutureStatus", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "FutureStatus",
                                      "%localid;\n"
                                      "                      %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Genre", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Genre",
                                      "%localid;\n"
                                      "                %formalname;\n"
                                      "                %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Geography",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Geography",
                                      "%localid;\n"
                                      "                    xml:lang CDATA  #IMPLIED\n"
                                      "                    %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "HeadLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "HeadLine",
                                      "%localid;\n"
                                      "                   xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Identification",
                                      "(NewsIdentifier , NameLabel? , DateLabel? , Label* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Identification",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "InsertAfter", "ANY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "InsertAfter",
                                      "%localid;\n"
                                      "                      DuidRef CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "InsertBefore", "ANY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "InsertBefore",
                                      "%localid;\n"
                                      "                       DuidRef CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Instruction",
                                      "(RevisionStatus* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Instruction",
                                      "%localid;\n"
                                      "                      %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "KeywordLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "KeywordLine",
                                      "%localid;\n"
                                      "                      xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Label",
                                      "(LabelType , LabelText )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Label", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "LabelText", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "LabelText", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "LabelType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "LabelType",
                                      "%localid;\n"
                                      "                    %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Language", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Language",
                                      "%localid;\n"
                                      "                   %formalname;\n"
                                      "                   %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Limitations",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Limitations",
                                      "%localid;\n"
                                      "                      xml:lang CDATA  #IMPLIED\n"
                                      "                      %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "MediaType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "MediaType",
                                      "%localid;\n"
                                      "                    %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Metadata",
                                      "(Catalog? , MetadataType , Property+ )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Metadata", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "MetadataType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "MetadataType",
                                      "%localid;\n"
                                      "                       %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "MimeType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "MimeType",
                                      "%localid;\n"
                                      "                   %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NameLabel", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NameLabel", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsComponent",
                                      "(Comment* , Catalog? , TopicSet* , Role? , BasisForChoice* , NewsLines? , AdministrativeMetadata? , RightsMetadata? , DescriptiveMetadata? , Metadata* ,  ( (NewsItem | NewsItemRef )+ | NewsComponent+ | ContentItem+ )? )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsComponent",
                                      "%localid;\n"
                                      "                        Essential        (yes | no )  'no'\n"
                                      "                        EquivalentsList  (yes | no )  'no'\n"
                                      "                        xml:lang        CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsEnvelope",
                                      "(TransmissionId? , SentFrom? , SentTo? , DateAndTime , NewsService* , NewsProduct* , Priority? )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsEnvelope", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsIdentifier",
                                      "(ProviderId , DateId , NewsItemId , RevisionId , PublicIdentifier )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsItem",
                                      "(Comment* , Catalog? , Identification , NewsManagement ,  (NewsComponent | Update+ | TopicSet )? )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsItem",
                                      "%localid;\n"
                                      "                   xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsItemId", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "MimeType",
                                      "Vocabulary CDATA  #IMPLIED\n"
                                      "                   Scheme     CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsItemRef",
                                      "(Comment* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "MimeType",
                                      "%localid;\n"
                                      "                   NewsItem CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsItemType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsItemType",
                                      "%localid;\n"
                                      "                       %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsLine",
                                      "(NewsLineType , NewsLineText+ )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsLine", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsLineText",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsLineText",
                                      "%localid;\n"
                                      "                       xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsLineType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsLineType",
                                      "%localid;\n"
                                      "                       %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsLines",
                                      "( (HeadLine , SubHeadLine? )* , ByLine* , DateLine* , CreditLine* , CopyrightLine* , RightsLine* , SeriesLine* , SlugLine* , KeywordLine* , NewsLine* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsLines", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsManagement",
                                      "(NewsItemType , FirstCreated , ThisRevisionCreated , Status , StatusWillChange? , Urgency? , RevisionHistory? , DerivedFrom* , AssociatedWith* , Instruction* , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsManagement",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsML",
                                      "(Catalog? , TopicSet* ,  (NewsEnvelope , NewsItem+ ) )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsML", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsProduct", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsProduct",
                                      "%localid;\n"
                                      "                      %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "NewsService", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "NewsService",
                                      "%localid;\n"
                                      "                      %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Notation", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Notation",
                                      "%localid;\n"
                                      "                   %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "OfInterestTo", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "OfInterestTo",
                                      "%localid;\n"
                                      "                       %formalname;\n"
                                      "                       %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Origin",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Origin",
                                      "%localid;\n"
                                      "                 %assignment;\n"
                                      "                 Href       CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Party", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Party",
                                      "%localid;\n"
                                      "                %formalname;\n"
                                      "                Topic      CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Priority", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Priority",
                                      "%localid;\n"
                                      "                   %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Property", "(Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Property",
                                      "%localid;\n"
                                      "                   %formalname;\n"
                                      "                   %assignment;\n"
                                      "                   Value         CDATA  #IMPLIED\n"
                                      "                   ValueRef      CDATA  #IMPLIED\n"
                                      "                   AllowedValues CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Provider", "(%party; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Provider", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "ProviderId", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "ProviderId",
                                      "Vocabulary CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "PublicIdentifier",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Relevance", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Relevance",
                                      "%localid;\n"
                                      "                    %formalname;\n"
                                      "                    %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Replace", "ANY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Replace",
                                      "%localid;\n"
                                      "                  DuidRef CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Resource",
                                      "(Urn? , Url* , DefaultVocabularyFor* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Resource", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RevisionHistory", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Origin",
                                      "%localid;\n"
                                      "                 Href    CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RevisionId", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "RevisionId",
                                      "PreviousRevision CDATA  #REQUIRED\n"
                                      "                     Update           CDATA  #REQUIRED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RevisionStatus",
                                      "(Status )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "RevisionStatus",
                                      "%localid;\n"
                                      "                         Revision CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RightsHolder",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "RightsHolder",
                                      "%localid;\n"
                                      "                       xml:lang   CDATA  #IMPLIED\n"
                                      "                       %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RightsLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "RightsLine",
                                      "%localid;\n"
                                      "                     xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "RightsMetadata",
                                      "(Catalog? , Copyright* , UsageRights* , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "RightsMetadata",
                                      "%localid;\n"
                                      "                         %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Role", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Role",
                                      "%localid;\n"
                                      "               %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SentFrom", "(%party; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SentFrom", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SentTo", "(%party; )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SentTo", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SeriesLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SeriesLine",
                                      "%localid;\n"
                                      "                     xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SizeInBytes", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SizeInBytes", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SlugLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SlugLine",
                                      "%localid;\n"
                                      "                   xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Source",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Source",
                                      "%localid;\n"
                                      "                 NewsItem CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "StartDate",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "StartDate",
                                      "%localid;\n"
                                      "                    xml:lang CDATA  #IMPLIED\n"
                                      "                    %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Status", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Status",
                                      "%localid;\n"
                                      "                 %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "StatusWillChange",
                                      "(FutureStatus , DateAndTime )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "StatusWillChange",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SubHeadLine",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SubHeadLine",
                                      "%localid;\n"
                                      "                      xml:lang CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Subject", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Subject",
                                      "%localid;\n"
                                      "                  %formalname;\n"
                                      "                  %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SubjectCode",
                                      "( (Subject | SubjectMatter | SubjectDetail ) , SubjectQualifier* )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SubjectCode",
                                      "%localid;\n"
                                      "                      %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SubjectDetail", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SubjectDetail",
                                      "%localid;\n"
                                      "                        %formalname;\n"
                                      "                        %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SubjectMatter", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SubjectMatter",
                                      "%localid;\n"
                                      "                        %formalname;\n"
                                      "                        %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SubjectQualifier", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SubjectQualifier",
                                      "%localid;\n"
                                      "                           %formalname;\n"
                                      "                           %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "SystemIdentifier",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "SystemIdentifier",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "ThisRevisionCreated",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "ThisRevisionCreated",
                                      "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Topic",
                                      "(Comment* , Catalog? , TopicType+ , FormalName* , Description* , Property* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Topic",
                                      "%localid;\n"
                                      "                Details CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TopicOccurrence", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TopicOccurrence",
                                      "%localid;\n"
                                      "                          %assignment;\n"
                                      "                          Topic      CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TopicSet",
                                      "(Comment* , Catalog? , TopicSetRef* , Topic* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TopicSet",
                                      "%localid;\n"
                                      "                   %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TopicSetRef",
                                      "(Comment* )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TopicSetRef",
                                      "%localid;\n"
                                      "                      TopicSet CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TopicType", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TopicType",
                                      "%localid;\n"
                                      "                    %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TopicUse", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TopicUse",
                                      "Topic   CDATA  #REQUIRED\n"
                                      "                   Context CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "TransmissionId",
                                      "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "TransmissionId",
                                      "%localid;\n"
                                      "                         Repeat  CDATA  #IMPLIED");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Update",
                                      "(InsertBefore | InsertAfter | Replace | Delete )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Update", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Urgency", "EMPTY");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Urgency",
                                      "%localid;\n"
                                      "                  %formalname;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Url", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Url", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "Urn", "(#PCDATA )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "Urn", "%localid;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "UsageRights",
                                      "(UsageType? , Geography? , RightsHolder? , Limitations? , StartDate? , EndDate? )");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "UsageRights",
                                      "%localid;\n"
                                      "                      %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    rc = xmlTextWriterWriteDTDElement(writer, "UsageType",
                                      "(#PCDATA | Origin )*");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDElement\n");
        return;
    }

    rc = xmlTextWriterWriteDTDAttlist(writer, "UsageType",
                                      "%localid;\n"
                                      "                    xml:lang CDATA  #IMPLIED\n"
                                      "                    %assignment;");
    if (rc < 0) {
        printf
            ("testXmlwriterDTD2: Error at xmlTextWriterWriteDTDAttlist\n");
        return;
    }

    /* since we did not start any documents or DTD's we must not
     * call the correspondent end functions. xmlFreeTextWriter
     * will flush the buffers for us or we could call
     * xmlTextWriterFlush. */
    xmlFreeTextWriter(writer);
}

#else
int
main(void)
{
    fprintf(stderr, "Writer or output support not compiled in\n");
    exit(1);
}
#endif
